import enum
from flask_login import UserMixin
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import Column, Enum, Integer, String, select, ForeignKey, Table, TIMESTAMP, Float
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import relationship, joinedload, declarative_base 
from werkzeug.security import check_password_hash, generate_password_hash

db = SQLAlchemy()
Base = declarative_base()
Base.query = db.session.query_property()


class students(UserMixin, db.Model):
    __tablename__ = "students"

    id = Column(Integer, primary_key=True, autoincrement=True)
    first_name = Column(String, nullable=False)
    last_name = Column(String, nullable=False)
    username = Column(String, nullable=False, unique=True)
    password = Column(String, nullable=False)
    
    # Constructor
    def __init__(self, first_name, last_name, username, password):
        self.first_name = first_name
        self.last_name = last_name
        self.username = username
        self.password = password

    @staticmethod
    def get_userinfo_by_username(username,password, db):
        try:
            query = select(students).where(students.username == username and students.password == password)
            return db.session.scalar(query)
        except SQLAlchemyError as e:
            print(f"Database error: {e}")

class teachers(UserMixin, db.Model):
    __tablename__ = "teachers"

    id = Column(Integer, primary_key=True, autoincrement=True)
    first_name = Column(String, nullable=False)
    last_name = Column(String, nullable=False)
    title = Column(String, nullable=False)
    username = Column(String, nullable=False, unique=True)
    password = Column(String, nullable=False)

    # Constructor
    def __init__(self, first_name, title, last_name, username, password):
        self.first_name = first_name
        self.last_name = last_name
        self.title = title
        self.username = username
        self.password = password

    @staticmethod
    def get_userinfo_by_username(username,password, db):
        try:
            stmt = select(teachers).where(teachers.username == username)
            user = db.session.scalar(stmt)
            is_valid = check_password_hash(user.password, password)
            if is_valid and user:
                return user
            return None
        except SQLAlchemyError as e:
            print(f"Database error: {e}")

    @staticmethod
    def getModules(teacherid, db):
        try:
            query = (
                select(ModuleORM)
                .join(TeacherModule, TeacherModule.c.module_id == ModuleORM.id)  # Join with junction table
                .where(TeacherModule.c.teacher_id == teacherid)  # Filter by teacher ID
            )
            return db.session.scalars(query).all()  # Fetch all matched modules
        except SQLAlchemyError as e:
            print(f"Database error: {e}")
            return None

# Junction table for Teachers and Modules
TeacherModule = Table(
    "teacher_module",
    Base.metadata,
    Column("teacher_id", Integer, ForeignKey("teachers.id", ondelete="CASCADE"), primary_key=True),
    Column("module_id", Integer, ForeignKey("modules.id", ondelete="CASCADE"), primary_key=True),
)

        
class Difficulty(enum.Enum):
    Easy = "Easy"
    Intermediate = "Intermediate"
    Hard = "Hard"

class LearningObjective(enum.Enum):
    Remember = "Remember"
    Understand = "Understand"
    Apply = "Apply"
    Analyse = "Analyse"
    Evaluate = "Evaluate"
    Create = "Create"

class FIB(db.Model):
    __tablename__ = "FIB"

    id = Column(Integer, primary_key=True, autoincrement=True)
    teacher_id = db.Column(db.Integer, db.ForeignKey("teachers.id"), nullable=False)
    module_id = db.Column(db.Integer, db.ForeignKey("modules.id"), nullable=False)
    question = Column(String, nullable=False)
    answer = Column(String, nullable=False)
    difficulty = Column(Enum(Difficulty, name="difficulty_enum"), nullable=False)
    learning_obj = Column(Enum(LearningObjective, name="learning_obj_enum"))
    feedback = db.Column(db.Text, nullable=True)
    active = db.Column(db.Boolean, default=True)

    # Constructor
    def __init__(self, teacher_id, module_id, question, answer, difficulty, learning_obj=None, feedback=None, active=True):
        self.teacher_id = teacher_id
        self.module_id = module_id
        self.question = question
        self.answer = answer
        self.difficulty = difficulty
        self.learning_obj = learning_obj
        self.feedback = feedback
        self.active = active


class MCQ(db.Model):
    __tablename__ = "MCQ"

    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    teacher_id = db.Column(db.Integer, db.ForeignKey("teachers.id"), nullable=False)
    module_id = db.Column(db.Integer, db.ForeignKey("modules.id"), nullable=False)
    question = db.Column(db.String, nullable=False)
    option_a = db.Column(db.String, nullable=False)
    option_b = db.Column(db.String, nullable=False)
    option_c = db.Column(db.String, nullable=False)
    option_d = db.Column(db.String, nullable=False)
    answer = db.Column(db.String, nullable=False)
    difficulty = db.Column(Enum(Difficulty, name="difficulty_enum"), nullable=False)
    learning_obj = db.Column(Enum(LearningObjective, name="learning_obj_enum"))
    feedback = db.Column(db.Text, nullable=True) 
    active = db.Column(db.Boolean, default=True)

    def __init__(self, teacher_id, module_id, question, option_a, option_b, option_c, option_d, answer, difficulty, active=True, learning_obj=None, feedback=None):
        self.teacher_id = teacher_id
        self.module_id = module_id
        self.question = question
        self.option_a = option_a
        self.option_b = option_b
        self.option_c = option_c
        self.option_d = option_d
        self.answer = answer
        self.difficulty = difficulty
        self.learning_obj = learning_obj
        self.feedback = feedback
        self.active = active


    def __repr__(self):
        return f'<MCQ {self.question}>'
    




# Kirs 
# Feedback review(Summative feedback) by CHRIS LYU
# class Student(db.Model):
#     id = db.Column(db.Integer, primary_key=True)
#     name = db.Column(db.String(100), nullable=False)
#     test_results = db.relationship('TestResult', backref='student', lazy=True)

class TestResult(db.Model):
    __tablename__ = 'test_result'
    id = db.Column(db.Integer, primary_key=True)
    student_id = db.Column(db.Integer, db.ForeignKey('students.id'), nullable=False)
    score = db.Column(db.Float, nullable=False)
    # feedback = db.relationship('Feedback', backref='test_result', lazy=True)
    # questions = db.relationship('QuestionAnswer', backref='test_result', lazy=True)

class Feedback(db.Model):
    __tablename__ = 'feedback'
    id = db.Column(db.Integer, primary_key=True)
    test_result_id = db.Column(db.Integer, db.ForeignKey('test_result.id'), nullable=False)
    comments = db.Column(db.Text, nullable=True)

class QuestionAnswer(db.Model):
    __tablename__ = 'question_answer'
    id = db.Column(db.Integer, primary_key=True)
    test_result_id = db.Column(db.Integer, db.ForeignKey('test_result.id'), nullable=False)
    question_text = db.Column(db.Text, nullable=False)
    student_answer = db.Column(db.Text, nullable=False)
    correct_answer = db.Column(db.Text, nullable=False)
# End of Teacher's feedback by CHRIS LYU

# Junction table for Students and Modules
StudentModule = Table(
    "student_module",
    Base.metadata,
    Column("student_id", Integer, ForeignKey("students.id", ondelete="CASCADE"), primary_key=True),
    Column("module_id", Integer, ForeignKey("modules.id", ondelete="CASCADE"), primary_key=True),
    Column("cohort", String, nullable=False)  # Cohort column (not an FK)
)

# Start of Module ORM and DAO
class Modules:
    def __init__(self, id, code, title):
        self.id = id
        self.code = code
        self.title = title


class ModuleORM(db.Model):
    __tablename__ = 'modules'
    id = db.Column(db.Integer, primary_key=True)
    code = db.Column(db.Text, nullable=False, unique=True)
    title = db.Column(db.Text, nullable=True)

    # One-to-Many Relationship with TestORM
    tests = relationship("TestORM", back_populates="module")

class ModuleDAO():
    def moduleById(moduleid, db):
        stm = select(ModuleORM).where(ModuleORM.id == moduleid)
        module = db.session.scalar(stm)
        return module

    def allModules(db):
        stm = select(ModuleORM)
        modules = db.session.scalars(stm).all()
        return modules
    
    @staticmethod
    def getCohortsByModule(moduleid, db):
        # Query cohorts from StudentModule (junction table)
        student_cohorts_query = (
            select(StudentModule.c.cohort)
            .where(StudentModule.c.module_id == moduleid)
        )

        # Query cohorts from TestORM table
        test_cohorts_query = (
            select(TestORM.cohort)  # Use TestORM
            .where(TestORM.module_id == moduleid)
        )

        # Execute queries and collect results
        student_cohorts = {row[0] for row in db.session.execute(student_cohorts_query)}
        test_cohorts = {row[0] for row in db.session.execute(test_cohorts_query)}

        # Return a list of distinct cohorts
        return list(student_cohorts | test_cohorts)  # Union to merge sets
    
# End of Module ORM and DAO

# Start of Test ORM and DAO
# Define the single TestORM model
class TestORM(db.Model):
    __tablename__ = 'tests'

    id = db.Column(db.Integer, primary_key=True)
    teacher_id = db.Column(db.Integer, db.ForeignKey('teachers.id'), nullable=False)
    module_id = db.Column(db.Integer, db.ForeignKey('modules.id'), nullable=False)
    cohort = db.Column(db.String, nullable=False)
    name = db.Column(db.String, nullable=False)
    instructions = db.Column(db.Text, nullable=True)
    date_created = db.Column(db.Date, nullable=False)
    type = db.Column(db.String, nullable=False)
    time_limit = db.Column(db.Integer, nullable=True)
    active = db.Column(db.Boolean, default=True)
    pass_percentage = db.Column(db.Float, nullable=False)

    # Relationship with TestQuestion (if needed)
    test_questions = db.relationship("TestQuestion", backref="test_parent", cascade="all, delete-orphan")

    #relationships with other tables
    module = relationship("ModuleORM", back_populates="tests")
    #teacher = relationship("TeacherORM", back_populates="tests")

    def __init__(self, teacher_id, module_id, cohort, name, instructions, date_created, type, time_limit, pass_percentage, active=True):
        self.teacher_id = teacher_id
        self.module_id = module_id
        self.cohort = cohort
        self.name = name
        self.instructions = instructions
        self.date_created = date_created
        self.type = type
        self.time_limit = time_limit
        self.pass_percentage = pass_percentage
        self.active = active

class TestQuestion(db.Model):
    __tablename__ = "test_questions"

    id = db.Column(db.Integer, primary_key=True, autoincrement=True)
    test_id = db.Column(db.Integer, db.ForeignKey("tests.id"), nullable=False)
    question_id = db.Column(db.Integer, nullable=False)
    question_type = db.Column(db.String, nullable=False)

    test = db.relationship("TestORM", backref=db.backref("test_questions_reverse", lazy="dynamic"))

    def __init__(self, test_id, question_id, question_type):
        self.test_id = test_id
        self.question_id = question_id
        self.question_type = question_type

# DAO for Test
class TestDAO:
    # Method to get Test by ID
    def get_test_by_id(test_id, db):
        stm = select(TestORM).where(TestORM.id == test_id)
        test = db.session.scalar(stm)
        return test

    # Method to get all tests
    def get_all_tests(db):
        stm = select(TestORM)
        tests = db.session.scalars(stm).all()
        return tests
    
    def getTestsByModuleAndCohort(module_id, cohort, db):
        # Query tests based on module_id and cohort
        stmt = select(TestORM.id, TestORM.name).where(
            (TestORM.module_id == module_id) & (TestORM.cohort == cohort)
        )

        # Execute query and return results as a list of dicts
        return [{"id": row[0], "name": row[1]} for row in db.session.execute(stmt)]

    # Method to create a new test
    def create_test(teacher_id, module_id, cohort, name, date_created, type, active, db):
        new_test = TestORM(
            teacher_id=teacher_id,
            module_id=module_id,
            cohort=cohort,
            name=name,
            date_created=date_created,
            type=type,
            active=active
        )
        db.session.add(new_test)
        db.session.commit()
        return new_test

    @staticmethod
    def getMCQQuestionsByTest(test_id, db):
        """
        Fetches all MCQ questions linked to a specific test ID.
        """
        stmt = (
            select(MCQ.id, MCQ.question, MCQ.option_a, MCQ.option_b, MCQ.option_c, MCQ.option_d, MCQ.answer, MCQ.difficulty, MCQ.learning_obj)
            .join(TestMCQ, TestMCQ.c.test_id == test_id)
            .where(MCQ.id == TestMCQ.c.mcq_id)
        )

        result = db.session.execute(stmt).all()
        return [
            {
                "id": row[0],
                "question": row[1],
                "options": [row[2], row[3], row[4], row[5]],
                "answer": row[6],
                "difficulty": str(row[7]) if row[7] else None,
                "learning_obj": str(row[8]) if row[8] else None,
            }
            for row in result
        ]
    
    def getFIBQuestionsByTest(test_id, db):
        """
        Fetches all FIB questions linked to a specific test ID.
        """
        stmt = (
            select(FIB.id, FIB.question, FIB.answer, FIB.difficulty, FIB.learning_obj)
            .join(TestFIB, TestFIB.c.test_id == test_id)
            .where(FIB.id == TestFIB.c.fib_id)
        )

        result = db.session.execute(stmt).all()
        return [
            {
                "id": row[0],
                "question": row[1],
                "answer": row[2],
                "difficulty": str(row[3]) if row[3] else None,
                "learning_obj": str(row[4]) if row[4] else None,
            }
            for row in result
        ]
    
    @staticmethod
    def getStudentsByTest(test_id, db):
        """
        Fetch students who are enrolled in the same module and cohort as the given test.
        """
        try:
            # Step 1: Get module_id and cohort for the test
            test_query = select(TestORM.module_id, TestORM.cohort).where(TestORM.id == test_id)
            test_result = db.session.execute(test_query).first()

            if not test_result:
                return []

            module_id, cohort = test_result

            # Step 2: Get students in the same module and cohort
            students_query = (
                select(students.id, students.first_name, students.last_name, students.username)
                .join(StudentModule, students.id == StudentModule.c.student_id)
                .where((StudentModule.c.module_id == module_id) & (StudentModule.c.cohort == cohort))
            )

            students_list = db.session.execute(students_query).all()

            # Step 3: Format and return student data
            return [
                {
                    "id": row[0],
                    "first_name": row[1],
                    "last_name": row[2],
                    "username": row[3]
                }
                for row in students_list
            ]

        except SQLAlchemyError as e:
            print(f"Database error: {e}")
            return []

    @staticmethod
    def getAttemptsByTest(test_id, db):
        stmt = select(TestAttempt).where(TestAttempt.test_id == test_id)
        attempts = db.session.execute(stmt).scalars().all()

        # Convert objects to dictionaries
        return [{"id": attempt.id, "test_id": attempt.test_id, "student_id": attempt.student_id, "score": attempt.score, "time_taken": attempt.time_taken, "attempt_date": attempt.attempt_date.isoformat() if attempt.attempt_date else None} for attempt in attempts]
        

# Junction table for Tests and MCQ Questions
TestMCQ = Table(
    "test_mcq",
    Base.metadata,
    Column("test_id", Integer, ForeignKey("test.id", ondelete="CASCADE"), primary_key=True),
    Column("mcq_id", Integer, ForeignKey("mcq.id", ondelete="CASCADE"), primary_key=True),
)

# Junction table for Tests and FIB Questions
TestFIB = Table(
    "test_fib",
    Base.metadata,
    Column("test_id", Integer, ForeignKey("test.id", ondelete="CASCADE"), primary_key=True),
    Column("fib_id", Integer, ForeignKey("fib.id", ondelete="CASCADE"), primary_key=True),
)


class TestAttempt(db.Model):
    __tablename__ = 'test_attempt'

    id = Column(Integer, primary_key=True, autoincrement=True)
    student_id = Column(Integer, ForeignKey('students.id'), nullable=False)
    test_id = Column(Integer, ForeignKey('tests.id'), nullable=False)
    score = Column(Float, nullable=False)
    time_taken = Column(Integer, nullable=False)
    attempt_date = Column(TIMESTAMP)

    # Relationships
    student = relationship("students", backref="test_attempt")
    test = relationship("TestORM", backref="test_attempt")

    def __init__(self, student_id, test_id, score, time_taken):
        self.student_id = student_id
        self.test_id = test_id
        self.score = score
        self.time_taken = time_taken

    @staticmethod
    def getQuestionAttempts(test_attempt_id, db):
        """
        Retrieves all question_attempt records related to a given test_attempt ID.
        """
        return db.session.scalars(
            select(QuestionAttempt).where(QuestionAttempt.test_attempt_id == test_attempt_id)
        ).all()

class QuestionAttempt(db.Model):
    __tablename__ = 'question_attempt'

    id = Column(Integer, primary_key=True, autoincrement=True)
    test_attempt_id = Column(Integer, ForeignKey('test_attempt.id'), nullable=False)
    mcq_id = Column(Integer, ForeignKey('MCQ.id'), nullable=True)  # Can be NULL if FIB is used
    fib_id = Column(Integer, ForeignKey('FIB.id'), nullable=True)  # Can be NULL if MCQ is used
    student_answer = Column(String, nullable=False)

    # Relationships
    test_attempt = relationship("TestAttempt", backref="question_attempts")
    mcq = relationship("MCQ", backref="question_attempts")
    fib = relationship("FIB", backref="question_attempts")

    def __init__(self, test_attempt_id, student_answer, mcq_id=None, fib_id=None):
        self.test_attempt_id = test_attempt_id
        self.student_answer = student_answer
        self.mcq_id = mcq_id
        self.fib_id = fib_id