// Code for initializing Quill rich text editor
// accessed 28-12-2024
// modified from Quill.js: https://quilljs.com/playground/snow

document.addEventListener("DOMContentLoaded", () => {
	// Initialize Quill
	const quill = new Quill('#editor', {
		modules: {
			toolbar: [
				[{ header: [1, 2, false] }],
				['bold', 'italic', 'underline'],
				['link', 'blockquote', 'code-block'],
				[{ list: 'ordered' }, { list: 'bullet' }],
			],
		},
		theme: 'snow',
	});

	// Prefill if question content exists
	const container = document.querySelector('#editor-container');
	if (container && container.dataset.question) {
		const htmlFromDB = container.dataset.question;
		quill.root.innerHTML = htmlFromDB;
	}

	// On form submit, store Quill HTML into hidden input
	const questionForm = document.querySelector('#questionform');
	if (questionForm) {
		questionForm.addEventListener('submit', (event) => {
			event.preventDefault();
			const questionInput = document.querySelector('#new_question');
			if (questionInput) {
				questionInput.value = quill.root.innerHTML;
			}
			questionForm.submit();
		});
	}
});


// Code for Review Test

$(document).ready(function () {

	let moduleDropdown = $("#module");
	let cohortDropdown = $("#cohort");
	let testDropdown = $("#test");
	let questionsDropdown = $("#questions");
	let studentsDropdown = $("#students");
	let difficultyDropdown = $("#difficulty");
	let objectiveDropdown = $("#objective");
	let attemptDropdown = $("#attemptType");

	let studentResultTable = $("#studentResult").DataTable({ // Initialize DataTable
		destroy: true, // Allows re-initialization
		responsive: true,
		paging: true,
		searching: true,
		ordering: true,
		columns: [
			{ title: "Student ID", data: "student_id" },
			{ title: "Forename", data: "forename" },
			{ title: "Surname", data: "surname" },
			{ title: "Score", data: "score" },
			{ title: "Time Taken", data: "time_taken" },
			{ title: "Attempt Date", data: "attempt_date", 
				render: function(data, type, row) {
                if (!data) return ""; // Handle null or empty values
                return data.split("T")[0]; // Extract only the date (YYYY-MM-DD)
            } }
		]
	});

	let studentAttemptTable = $("#studentAttemptResults").DataTable({ // Initialize DataTable
		destroy: true, // Allows re-initialization
		responsive: true,
		paging: true,
		searching: true,
		ordering: true,
		columns: [
			{ title: "Attempt", data: "attempt_id" },
			{ title: "Score", data: "score" },
			{ title: "Time Taken", data: "time_taken" },
			{ title: "Attempt Date", data: "attempt_date", 
				render: function(data, type, row) {
                if (!data) return ""; // Handle null or empty values
                return data.split("T")[0]; // Extract only the date (YYYY-MM-DD)
            } }
		]
	});

	$('#studentResult tbody').on('click', 'tr', function () {
		let rowData = studentResultTable.row(this).data();
		if (rowData) {
			let fullName = `${rowData.forename} ${rowData.surname}`;
			let studentId = rowData.student_id;
			let testId = $('#test').val(); // Assuming this is still in your page context
	
			// Redirect using query params
			window.location.href = `/reviewStudents?student_id=${studentId}&test_id=${testId}&name=${encodeURIComponent(fullName)}`;
		}
	});

	// When difficulty or learning objective changes, filter questions based on selected values
	function filterQuestions() {
		let selectedDifficulties = difficultyDropdown.val(); // Get selected difficulties
		let selectedObjectives = objectiveDropdown.val(); // Get selected learning objectives
		let selectedQuestionIds = [];
		let selectedQuestionTypes = [];

		questionsDropdown.find('option').each(function () {
			let questionDifficulty = $(this).data('difficulty');
			let questionObjective = $(this).data('objective');
			let questionType = $(this).data('qtype');
			let questionId = $(this).val();

			// Check if the question matches the selected difficulty and learning objective
			if ((selectedDifficulties && selectedDifficulties.includes(questionDifficulty)) &&
				(selectedObjectives && selectedObjectives.includes(questionObjective))) {
				$(this).show(); // Show matching question
				$(this).prop('selected', true);
				selectedQuestionIds.push(questionId); // Collect the selected question IDs
				selectedQuestionTypes.push(questionType);

			} else {
				$(this).hide(); // Hide non-matching question
				$(this).prop('selected', false);
			}
		});

		$(".selectpicker").selectpicker('refresh'); // Refresh selectpicker to apply changes
		recalculateScores();

	}

	// When difficulty changes, filter questions based on selected difficulties
	difficultyDropdown.change(function () {
		filterQuestions(); // Reapply filters when difficulty is changed
	});

	// When learning objective changes, filter questions based on selected objectives
	objectiveDropdown.change(function () {
		filterQuestions(); // Reapply filters when learning objective is changed
	});

	questionsDropdown.change(function () {
		recalculateScores();
	});

	// When module changes, reset cohort & test dropdowns
	moduleDropdown.change(function () {
		let moduleId = $(this).val();

		// Reset cohort, test, questions & students dropdowns
		cohortDropdown.prop("disabled", true).html('<option value="" disabled selected>Select a Cohort</option>');
		testDropdown.prop("disabled", true).html('<option value="" disabled selected></option>');
		questionsDropdown.prop("disabled", true).selectpicker('deselectAll');
		studentsDropdown.prop("disabled", true).selectpicker('deselectAll');
		difficultyDropdown.prop("disabled", true).selectpicker('deselectAll');
		objectiveDropdown.prop("disabled", true).selectpicker('deselectAll');
		attemptDropdown.prop("disabled", true).val('hidden');


		$(".selectpicker").selectpicker('refresh');
		// **CLEAR TABLE ENTRIES** before loading new data
		studentResultTable.clear().draw();
		$("#difficulty").prop("disabled", true);
		$("#objective").prop("disabled", true);
		$("#onlyAttempts").prop("disabled", true);
		$("#onlyAttempts").prop("checked", false);
		$("#score").text("N/A"); // Reset Score
		$("#timetaken").text("N/A"); // Reset Time Taken

		if (moduleId) {
			// Fetch cohorts for the selected module
			$.ajax({
				url: "/getCohorts",
				type: "POST",
				contentType: "application/json",
				data: JSON.stringify({ module_id: moduleId }),
				success: function (response) {
					if (response.length > 0) {
						response.forEach(cohort => {
							cohortDropdown.append(`<option value="${cohort}">${cohort}</option>`);
						});
						cohortDropdown.prop("disabled", false);
					}
					else {
						cohortDropdown.html('<option value="">No Cohorts available</option>').prop("disabled", true);
						testDropdown.html('<option value=""></option>').prop("disabled", true);
					}
				}
			});
		}
	});

	// When cohort changes, fetch tests for the selected module and cohort
	cohortDropdown.change(function () {
		let moduleId = moduleDropdown.val();
		let cohort = $(this).val();

		// Reset test, questions & students dropdowns
		testDropdown.prop("disabled", true).html('<option value="" disabled selected>Select a Test</option>');
		questionsDropdown.prop("disabled", true).selectpicker('deselectAll');
		studentsDropdown.prop("disabled", true).selectpicker('deselectAll');
		attemptDropdown.prop("disabled", true).val('hidden');
		difficultyDropdown.prop("disabled", true).selectpicker('deselectAll');
		objectiveDropdown.prop("disabled", true).selectpicker('deselectAll');
		questionsDropdown.html('<option value="" disabled selected></option>');
		questionsDropdown.prop("disabled", true);
		studentsDropdown.html('<option value="" disabled selected></option>');
		studentsDropdown.prop("disabled", true);
		$(".selectpicker").selectpicker('refresh');
		// **CLEAR TABLE ENTRIES** before loading new data
		studentResultTable.clear().draw();
		$("#difficulty").prop("disabled", true);
		$("#objective").prop("disabled", true);
		$("#onlyAttempts").prop("disabled", true);
		$("#onlyAttempts").prop("checked", false);
		$("#score").text("N/A"); // Reset Score
		$("#timetaken").text("N/A"); // Reset Time Taken

		if (moduleId && cohort) {
			// Fetch tests for the selected module & cohort
			$.ajax({
				url: "/getTests",
				type: "POST",
				contentType: "application/json",
				data: JSON.stringify({ module_id: moduleId, cohort: cohort }),
				success: function (response) {
					if (response.length > 0) {
						response.forEach(test => {
							testDropdown.append(`<option value="${test.id}">${test.name}</option>`);
						});
						testDropdown.prop("disabled", false);
					}
					else {
						testDropdown.html('<option value="">No Tests available</option>').prop("disabled", true);
					}
				}
			});
		}
	});

	// When test changes, fetch questions and students for the selected test
	testDropdown.change(function () {
		let testId = $(this).val();

		// Reset questions dropdown
		questionsDropdown.prop("disabled", true).html('<option value="" disabled selected>No Questions Available</option>');
		studentsDropdown.prop("disabled", true).html('<option value="" disabled selected>No Students Available</option>');
		$("#onlyAttempts").prop("disabled", true);
		$("#onlyAttempts").prop("checked", false);
		$("#difficulty").prop("disabled", true);
		$("#objective").prop("disabled", true);
		difficultyDropdown.prop("disabled", true).selectpicker('deselectAll');
		objectiveDropdown.prop("disabled", true).selectpicker('deselectAll');
		attemptDropdown.prop("disabled", true).val('hidden');
		
		$(".selectpicker").selectpicker('refresh');
		// **CLEAR TABLE ENTRIES** before loading new data
		studentResultTable.clear().draw();
		$("#score").text("N/A"); // Reset Score
		$("#timetaken").text("N/A"); // Reset Time Taken

		if (testId) {
			// Fetch tests for the selected module & cohort
			$.ajax({
				url: "/getTestDetails",
				type: "POST",
				contentType: "application/json",
				data: JSON.stringify({ test_id: testId }),
				success: function (response) {
					questionsDropdown.html('');
					studentsDropdown.html('');
					if (response.mcq_questions.length > 0 || response.fib_questions.length > 0) {
						response.mcq_questions.forEach(mcq => {
							questionsDropdown.append(`<option value="${mcq.id}" data-difficulty="${mcq.difficulty}" data-objective="${mcq.learning_obj}" data-qtype="mcq">${mcq.question}</option>`);
						});
						response.fib_questions.forEach(fib => {
							questionsDropdown.append(`<option value="${fib.id}" data-difficulty="${fib.difficulty}" data-objective="${fib.learning_obj}" data-qtype="fib">${fib.question}</option>`);
						});
						response.students.forEach(student => {
							studentsDropdown.append(`<option value="${student.id}">${student.first_name} ${student.last_name}</option>`);
							// Find student attempt
							let attempt = response.test_attempts.find(attempt => attempt.student_id === student.id);

							let score = 0;
							let total = 0;
							if (attempt) {
								$.ajax({
									url: "/getStudentQuestions",
									type: "POST",
									contentType: "application/json",
									data: JSON.stringify({ test_attempt_id: attempt.id, test_id: testId }),
									success: function (response) {
										response.question_attempts.forEach(question => {
											total += 1;
											if (question.student_answer == question.correct_answer) {
												score += 1;
											}
										})
										score = (score / total) * 100;
										if (isNaN(score)) {
											score = 0;
										}

										// Ensure the data object contains exactly 6 fields
										let rowData = {
											student_id: student.id,
											forename: student.first_name,
											surname: student.last_name,
											score: attempt ? Math.round(score * 100) / 100 : '0',
											time_taken: attempt ? attempt.time_taken : '0',
											attempt_date: attempt ? attempt.attempt_date : 'N/A'
										};
										// Add data to DataTable
										studentResultTable.rows().every(function () {
											var existingRowData = this.data();  // Get the current row's data
											if (!rowData === existingRowData) {
												studentResultTable.row.add(rowData).draw(false);
											}
										});
									}
								})
							}
							else {
								// Ensure the data object contains exactly 6 fields
								let rowData = {
									student_id: student.id,
									forename: student.first_name,
									surname: student.last_name,
									score: 'N/A',
									time_taken: 'N/A',
									attempt_date: 'N/A'
								};
								// Add data to DataTable
								studentResultTable.row.add(rowData).draw(false);
							}

						});

						questionsDropdown.prop("disabled", false);
						studentsDropdown.prop("disabled", false);
						$("#difficulty").prop("disabled", false);
						$("#objective").prop("disabled", false);
						$("#onlyAttempts").prop("disabled", false);
						attemptDropdown.prop("disabled", false).val('Best');
						$('#attemptType option[value="hidden"]').hide();
						$(".selectpicker").selectpicker('refresh');
						$(".selectpicker").selectpicker('selectAll');
						

						// **Update Average Score & Time Taken**
						updateScoreAndTimeFromTable();
					}
					else {
						questionsDropdown.prop("disabled", true).html('<option value="" disabled selected>No Questions Available</option>');
						studentsDropdown.prop("disabled", true).html('<option value="" disabled selected>No Students Available</option>');
						$(".selectpicker").selectpicker('refresh');
					}
				}
			});
		}
		else {
			questionsDropdown.html('<option value="" disabled selected>No Questions available</option>');
			questionsDropdown.prop("disabled", true);
			$(".selectpicker").selectpicker('refresh');
		}
	});

	studentsDropdown.change(function () {
		recalculateScores();
	});

	// **Function to get Average Score and Time Taken from DataTable**
	function updateScoreAndTimeFromTable() {
		let totalScore = 0;
		let totalTimeTaken = 0;
		let studentCount = 0;

		// Iterate through all the rows in the DataTable
		studentResultTable.rows().every(function () {
			let rowData = this.data();
			let score = parseFloat(rowData.score);
			let timeTaken = parseFloat(rowData.time_taken);

			// Ensure that the score and timeTaken are valid numbers
			// Skip "N/A", null, or non-numeric values
			if (!isNaN(score) && score !== null && score !== 'N/A') {
				totalScore += score;
				studentCount++;
			}
			if (!isNaN(timeTaken) && timeTaken !== null && timeTaken !== 'N/A') {
				totalTimeTaken += timeTaken;
			}
		});

		// Calculate the average score and time taken
		let averageScore = studentCount > 0 ? (totalScore / studentCount).toFixed(2) : 0;
		let averageTimeTaken = studentCount > 0 ? (totalTimeTaken / studentCount).toFixed(2) : 0;

		// Update the #score and #timetaken elements
		$("#score").text(`${averageScore}`);
		$("#timetaken").text(`${averageTimeTaken}`);
	}

	// Optional: Call this function whenever the table is drawn
	studentResultTable.on('draw', function () {
		updateScoreAndTimeFromTable();
	});

	// Function to recalculate the score for each student based on selected questions
	function recalculateScores() {
		studentResultTable.clear().draw(); // Clear the table before updating it
		let testId = testDropdown.val();
		let selectedMCQQuestionIds = [];
		questionsDropdown.find('option:selected').each(function () {
			// Check if the data-qtype attribute is equal to 'mcq'
			if ($(this).data('qtype') === 'mcq') {
				selectedMCQQuestionIds.push($(this).val());
			}
		});
		let selectedFIBQuestionIds = [];
		questionsDropdown.find('option:selected').each(function () {
			// Check if the data-qtype attribute is equal to 'fib'
			if ($(this).data('qtype') === 'fib') {
				selectedFIBQuestionIds.push($(this).val());
			}
		});
		let selectedStudentIds = [];
		studentsDropdown.find('option:selected').each(function () {
			selectedStudentIds.push($(this).val());
		});

		let attempt_type = attemptDropdown.val();

		$.ajax({
			url: "/getTestDetails",
			type: "POST",
			contentType: "application/json",
			data: JSON.stringify({ test_id: testId, attempt_type: attempt_type}),
			success: function (response) {
				response.students.forEach(student => {
					if (selectedStudentIds.includes(student.id.toString())) {
						// Find student attempt
						let attempt = response.test_attempts.find(attempt => attempt.student_id === student.id);

						let score = 0;
						let total = 0;
						if (attempt) {
							$.ajax({
								url: "/getStudentQuestions",
								type: "POST",
								contentType: "application/json",
								data: JSON.stringify({ test_attempt_id: attempt.id, test_id: testId }),
								success: function (response) {
									response.question_attempts.forEach(question => {
										if (question.question_type === "mcq") {
											//console.log(selectedMCQQuestionIds.includes(question.id+""));
											if (selectedMCQQuestionIds.includes(question.id + "")) {
												total += 1;
												if (question.student_answer == question.correct_answer) {
													score += 1;
												}
											}
										}
										if (question.question_type === "fib") {
											if (selectedFIBQuestionIds.includes(question.id + "")) {
												total += 1;
												if (question.student_answer == question.correct_answer) {
													score += 1;
												}
											}
										}
									})
									score = (score / total) * 100;
									if (isNaN(score)) {
										score = 0;
									}

									// Ensure the data object contains exactly 6 fields
									let rowData = {
										student_id: student.id,
										forename: student.first_name,
										surname: student.last_name,
										score: attempt ? Math.round(score * 100) / 100 : '0',
										time_taken: attempt ? attempt.time_taken : '0',
										attempt_date: attempt ? attempt.attempt_date : 'N/A'
									};
									// Add data to DataTable
									studentResultTable.row.add(rowData).draw(false);
								}
							})
						}
						else {
							// Ensure the data object contains exactly 6 fields
							let rowData = {
								student_id: student.id,
								forename: student.first_name,
								surname: student.last_name,
								score: 'N/A',
								time_taken: 'N/A',
								attempt_date: 'N/A'
							};
							// Add data to DataTable
							studentResultTable.row.add(rowData).draw(false);
						}
					}
				})
			}
		})
	}
	let exportButton = $(".btn-primary[type='export']");

	// Function to check if DataTable is empty and enable/disable Export Button
	function toggleExportButton() {
		exportButton.prop("disabled", studentResultTable.rows().count() === 0);
	}

	// Call the function whenever the DataTable is updated
	studentResultTable.on('draw', toggleExportButton);

	exportButton.click(function (event) {
		event.preventDefault();
		let selectedQuestions = [];

		// Get selected questions and their IDs
		questionsDropdown.find('option:selected').each(function () {
			selectedQuestions.push({
				id: $(this).val(),
				question: $(this).text(),
				type: $(this).data('qtype'),
			});
		});

		let studentData = [];
		// Collect data from the DataTable
		studentResultTable.rows().every(function () {
			let rowData = this.data();
			studentData.push({
				student_id: rowData.student_id,
				forename: rowData.forename,
				surname: rowData.surname,
				score: rowData.score,
				time_taken: rowData.time_taken,
				attempt_date: rowData.attempt_date,
				answers: {} // Initialize answers as an empty object
			});
		});

		let testId = testDropdown.val();

		// Fetch test details
		$.ajax({
			url: "/getTestDetails",
			type: "POST",
			contentType: "application/json",
			data: JSON.stringify({ test_id: testId })
		}).done(function (response) {
			let ajaxCalls = [];

			response.students.forEach(studentAnswer => {
				let attempt = response.test_attempts.find(attempt => attempt.student_id === studentAnswer.id);
				if (attempt) {
					// Store AJAX calls in an array
					let ajaxCall = $.ajax({
						url: "/getStudentQuestions",
						type: "POST",
						contentType: "application/json",
						data: JSON.stringify({ test_attempt_id: attempt.id, test_id: testId })
					}).done(function (response) {
						response.question_attempts.forEach(question => {
							let student = studentData.find(s => s.student_id == studentAnswer.id);
							if (student) {
								let answerKey = `${question.id}_${question.question_type}`;
								student.answers[answerKey] = {
									student_answer: question.student_answer,
									correct_answer: question.correct_answer
								};
							}
						});
					});

					ajaxCalls.push(ajaxCall);
				}
			});

			// Wait for all AJAX calls to finish before calling generateExcel
			$.when.apply($, ajaxCalls).done(function () {
				generateExcel(studentData, selectedQuestions);
			});
		});
	});

	function generateExcel(studentData, selectedQuestions) {
		let wb = XLSX.utils.book_new();
		let wsData = [];

		// Header Row
		let headers = ["Student ID", "Forename", "Surname", "Score", "Time Taken", "Attempt Date"];
		selectedQuestions.forEach(q => {
			headers.push(q.question, "Correct Answer");
		});
		wsData.push(headers);

		// Data Rows
		studentData.forEach(student => {
			let row = [
				student.student_id,
				student.forename,
				student.surname,
				student.score,
				student.time_taken,
				student.attempt_date
			];

			selectedQuestions.forEach(q => {
				let answerKey = `${q.id}_${q.type}`;
				let answerData = student.answers[answerKey] || { student_answer: "N/A", correct_answer: "N/A" };

				row.push(answerData.student_answer, answerData.correct_answer);
			});
			wsData.push(row);
		});

		// Create worksheet from wsData
		let ws = XLSX.utils.aoa_to_sheet(wsData);

		// Apply cell formatting

		// 1. Set column widths (make question columns wider)
		let colWidths = [
			{ wpx: 100 },  // Student ID
			{ wpx: 120 },  // Forename
			{ wpx: 120 },  // Surname
			{ wpx: 60 },   // Score
			{ wpx: 90 },   // Time Taken
			{ wpx: 120 },  // Attempt Date
		];

		selectedQuestions.forEach(() => {
			colWidths.push({ wpx: 250 });  // Question column (width for questions)
			colWidths.push({ wpx: 150 });  // Correct Answer column (width for correct answers)
		});

		ws['!cols'] = colWidths;  // Apply column widths to the worksheet

		// 2. Apply borders to all cells and middle alignment
		const range = XLSX.utils.decode_range(ws['!ref']);  // Get the range of the worksheet
		for (let row = range.s.r; row <= range.e.r; row++) {
			for (let col = range.s.c; col <= range.e.c; col++) {
				let cellAddress = { r: row, c: col };
				let cellRef = XLSX.utils.encode_cell(cellAddress);

				if (!ws[cellRef]) ws[cellRef] = {};  // Ensure cell exists

				// Apply border to the cell
				ws[cellRef].s = {
					border: {
						top: { style: 'thin', color: { rgb: "000000" } },
						left: { style: 'thin', color: { rgb: "000000" } },
						bottom: { style: 'thin', color: { rgb: "000000" } },
						right: { style: 'thin', color: { rgb: "000000" } },
					},
					alignment: {
						vertical: 'center', // Middle align vertically
						horizontal: 'center', // Center align horizontally
						wrapText: true // Enable text wrapping
					}
				};
			}
		}

		// 3. Apply text wrapping and reduce row height
		for (let row = range.s.r; row <= range.e.r; row++) {
			let rowRef = XLSX.utils.encode_row(row); // Get row reference
			if (!ws['!rows']) ws['!rows'] = [];
			ws['!rows'][row] = { hpx: 24 };  // Reduced row height (adjust as needed)
		}

		// 4. Add a bottom border to the question and correct answer headers
		selectedQuestions.forEach((q, idx) => {
			let questionColIndex = 6 + idx * 2;  // Column index for "Question"
			let correctAnswerColIndex = 7 + idx * 2;  // Column index for "Correct Answer"

			// Add bottom border to the "Question" and "Correct Answer" header cells
			let questionCellRef = XLSX.utils.encode_cell({ r: 0, c: questionColIndex });
			let correctAnswerCellRef = XLSX.utils.encode_cell({ r: 0, c: correctAnswerColIndex });

			if (!ws[questionCellRef]) ws[questionCellRef] = {};  // Ensure cell exists
			if (!ws[correctAnswerCellRef]) ws[correctAnswerCellRef] = {};  // Ensure cell exists

			// Apply bottom border to the header cells
			ws[questionCellRef].s = {
				border: {
					bottom: { style: 'thin', color: { rgb: "000000" } }
				}
			};
			ws[correctAnswerCellRef].s = {
				border: {
					bottom: { style: 'thin', color: { rgb: "000000" } }
				}
			};
		});

		// 5. Set the sheet name
		XLSX.utils.book_append_sheet(wb, ws, "Student Results");

		// Export file
		XLSX.writeFile(wb, "Student_Results.xlsx");
	}

	$("#onlyAttempts").change(function () {
        if ($(this).is(":checked")) {
            filterStudentsWithAttempts(true);
        } else {
            filterStudentsWithAttempts(false);
        }
    });

	function filterStudentsWithAttempts(checked) {
		
			let studentsWithAttempt = [];
			
			let testId = testDropdown.val();
			$.ajax({
				url: "/getTestDetails",
				type: "POST",
				contentType: "application/json",
				data: JSON.stringify({ test_id: testId }),
				success: function (response) {
					response.students.forEach(student => {
						response.test_attempts.forEach(attempt => {
							if(student.id === attempt.student_id) {
								
								studentsWithAttempt.push(student.id.toString());
						
							}

						})
					})
					if (checked) {
						studentsDropdown.find('option').each(function () {
							let studentId = $(this).val();
							if (!studentsWithAttempt.includes(studentId.toString())) {
								$(this).hide();
								$(this).prop('selected', false);
								$(".selectpicker").selectpicker('refresh');
							}
						})
						recalculateScores();
					}
					else {
						studentsDropdown.find('option').each(function () {
							$(this).show();
							$(this).prop('selected', true);
							$(".selectpicker").selectpicker('refresh');
						})
						recalculateScores();
					}
				}
			})
	}

	$("#attemptType").change(function () {
		recalculateScores();
	});


});